
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <resolv.h>
#include "res_internal.h"

static struct hostent *fake_hostent(const char *hostname, unsigned long addr,
				    struct hostent_answer *result);
static struct hostent *file_find(const char *name, struct res_data *data,
				 struct hostent_answer *result);

struct hostent *gethostbyname(const char *hostname)
{
    struct res_data *data;

    data = _res_init();
    return (data) ? gethostbyname_r(hostname, &data->host_answer) : NULL;
}

struct hostent *gethostbyname_r(const char *hostname,
				struct hostent_answer *result)
{
    struct res_data *data;
    unsigned long addr;
    querybuf buf;
    const char *p;
    int n;

    data = _res_init();
    if (!data)
	return NULL;

    /* Check for all-numeric hostname with no trailing dot. */
    if (isdigit(p[0])) {
	p = hostname;
	while (*p && (isdigit(*p) || *p == '.'))
	    p++;
	if (!*p && p[-1] != '.') {
	    /* Looks like an IP address; convert it. */
	    addr = inet_addr(hostname);
	    if (addr == -1) {
		data->errval = HOST_NOT_FOUND;
		return NULL;
	    }
	    return fake_hostent(hostname, addr, result);
	}
    }

    /* Do the search. */
    n = res_search(hostname, C_IN, T_A, buf.buf, sizeof(buf));
    if (n >= 0)
	return _res_parse_answer(&buf, n, 0, result, data);
    else if (errno == ECONNREFUSED)
	return file_find(hostname, data, result);
    else
	return NULL;
}

static struct hostent *fake_hostent(const char *hostname, unsigned long addr,
				    struct hostent_answer *result)
{
    strncpy(result->name, hostname, BUFSIZ - 1);
    result->name[BUFSIZ - 1] = 0;
    result->host.h_name = result->name;

    result->host_addr = addr;
    result->h_addr_ptrs[0] = (char *) &result->host_addr;
    result->h_addr_ptrs[1] = NULL;
    result->host.h_addr_list = result->h_addr_ptrs;
    result->host.h_length = sizeof(unsigned long);

    result->host_aliases[0] = NULL;
    result->host.h_aliases = result->host_aliases;

    result->host.h_addrtype = AF_INET;

    return &result->host;
}

static struct hostent *file_find(const char *name, struct res_data *data,
				 struct hostent_answer *result)
{
    struct hostent *entry;
    char **alias;

    _sethtent(data, 0);
    while (1) {

	/* Get a host entry friom the file. */
	entry = _gethtent(data, result);
	if (!entry)
	    break;

	/* Check the name and aliases. */
	if (strcasecmp(entry->h_name, name) == 0)
	    break;
	for (alias = entry->h_aliases; *alias; alias++) {
	    if (strcasecmp(alias, name) == 0)
		break;
	}
	if (*alias)
	    break;
    }
    _endhtent(data);

    if (!entry)
	data->errval = HOST_NOT_FOUND;
    return entry;
}

